import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.util.Set;

public class Board {

	private HashMap pieces;

	public Board() {
		pieces = new HashMap();
	}
	
	public Board (Board t) {
		pieces = new HashMap();
		
		Vector ps = t.getPiecesVector();
		for (int i=0; i < 32; i++) {
			if (ps.elementAt (i) != null) {
				pieces.put (new Integer (i+1), ps.elementAt(i));
			} else {
				pieces.put (new Integer (i+1), null);
			}
		}
	}
	
	public Vector getPiecesVector () {
		Vector ps = new Vector();
		
		for (int i=1; i < 33; i++) {
			Piece p = null;
			if (pieces.get (new Integer(i)) != null) {
				p = new Piece ((Piece) pieces.get (new Integer (i)));
			}			
			ps.add (p);
		}
		return ps;
	}
	
	public int numPieces (int playerNum) {
		return playersPieces(playerNum).size();
	}
	
	public Vector playersPieces (int playerNum) {
		Vector ps = new Vector();
		
		for (int i=1; i < 33; i++) {
			Piece p = (Piece) pieces.get (new Integer (i));
			if (p != null) {
				 if (p.ownedBy (playerNum)) {
				 	ps.add (p);
				 }
			}
		}
		
		return ps;
	}

	public Vector forcedJumps (int playerNum) {
		Vector jumps = new Vector();
		
		Vector myPieces = playersPieces (playerNum);
		
		for (int i = 0; i < myPieces.size(); i++) {
			Piece p = (Piece) myPieces.elementAt (i);
			jumps.addAll (jumpsForPiece (p, playerNum));
		}
		
		return jumps;
	}
	
	public Vector jumpsForPiece (Piece p, int playerNum) {
		Vector jumps = new Vector();
		
		if (p.isKing()) {
			if (jumpedPiece (p.getLocation(), p.getLocation() + 9) != null && !jumpWraps (p.getLocation(), p.getLocation() + 9)) {
				jumps.add (p.getLocation() + "-" + (p.getLocation() + 9));
			}
			if (jumpedPiece (p.getLocation(), p.getLocation() + 7) != null && !jumpWraps (p.getLocation(), p.getLocation() + 7)) {
				jumps.add (p.getLocation() + "-" + (p.getLocation() + 7));
			}
			if (jumpedPiece (p.getLocation(), p.getLocation() - 9) != null && !jumpWraps (p.getLocation(), p.getLocation() - 9)) {
				jumps.add (p.getLocation() + "-" + (p.getLocation() - 9));
			}
			if (jumpedPiece (p.getLocation(), p.getLocation() - 7) != null && !jumpWraps (p.getLocation(), p.getLocation() - 7)) {
				jumps.add (p.getLocation() + "-" + (p.getLocation() - 7));
			}
			return jumps;
		}
		
		if (p.getOwner() == 0) {
			if (jumpedPiece (p.getLocation(), p.getLocation() + 9) != null && !jumpWraps (p.getLocation(), p.getLocation() + 9)) {
				jumps.add (p.getLocation() + "-" + (p.getLocation() + 9));
			}
			if (jumpedPiece (p.getLocation(), p.getLocation() + 7) != null && !jumpWraps (p.getLocation(), p.getLocation() + 7)) {
				jumps.add (p.getLocation() + "-" + (p.getLocation() + 7));
			}
		} else {
			if (jumpedPiece (p.getLocation(), p.getLocation() - 9) != null && !jumpWraps (p.getLocation(), p.getLocation() - 9)) {
				jumps.add (p.getLocation() + "-" + (p.getLocation() - 9));
			}
			if (jumpedPiece (p.getLocation(), p.getLocation() - 7) != null && !jumpWraps (p.getLocation(), p.getLocation() - 7)) {
				jumps.add (p.getLocation() + "-" + (p.getLocation() - 7));
			}
		}
		
		return jumps;
	}		
	

	public Vector movesForPiece (Piece p, int playerNum) {
		Vector moves = new Vector();
		
		moves.addAll (jumpsForPiece (p, playerNum));
		
		if (moves.size() == 0) {
			if (validMove (p.getLocation(), p.getLocation() + 3, playerNum) && !moveWraps (p.getLocation(), p.getLocation() +3)) {
				moves.add (p.getLocation() + "-" + (p.getLocation() + 3));
			}
			if (validMove (p.getLocation(), p.getLocation() + 4, playerNum) && !moveWraps (p.getLocation(), p.getLocation() + 4)) {
				moves.add (p.getLocation() + "-" + (p.getLocation() + 4));
			}
			if (validMove (p.getLocation(), p.getLocation() + 5, playerNum) && !moveWraps (p.getLocation(), p.getLocation() + 5)) {
				moves.add (p.getLocation() + "-" + (p.getLocation() + 5));
			}
			if (validMove (p.getLocation(), p.getLocation() - 3, playerNum) && !moveWraps (p.getLocation(), p.getLocation() - 3)) {
				moves.add (p.getLocation() + "-" + (p.getLocation() - 3));
			}
			if (validMove (p.getLocation(), p.getLocation() - 4, playerNum) && !moveWraps (p.getLocation(), p.getLocation() - 4)) {
				moves.add (p.getLocation() + "-" + (p.getLocation() - 4));
			}
			if (validMove (p.getLocation(), p.getLocation() - 5, playerNum) && !moveWraps (p.getLocation(), p.getLocation() - 5)) {
				moves.add (p.getLocation() + "-" + (p.getLocation() - 5));
			}
		}
		
		return moves;
	}
	
	public Vector legalMoves (int playerNum) {
		Vector moves = new Vector();
		
		Vector myPieces = playersPieces (playerNum);
		
		for (int i=0; i < myPieces.size(); i++) {
			Piece p = (Piece) myPieces.elementAt (i);
			moves.addAll (movesForPiece (p, playerNum));
		}
			
		return moves;
	}
	
	public boolean legalMovesLeft(int playerNum)
    {
        Vector moves = legalMoves(playerNum);
        if (moves.isEmpty())
            return false;
        else
            return true;
    }

	
	public boolean jumpWraps (int place1, int place2) {
		if (place2 < 1) {
			return true;
		}
		if (place2 > 32) {
			return true;
		}
		if (place1 % 8 == 0) {
			if (Math.abs (place2 - place1) == 9 && place1 < place2) {
				return true;
			} else if (Math.abs (place2 - place1) == 7 && place1 > place2) {
				return true;
			}
		} else if (place1 % 8 == 4) {
			if (Math.abs (place2 - place1) == 9 && place1 < place2) {
				return true;
			} else if (Math.abs (place2 - place1) == 7 && place1 > place2) {
				return true;
			}
		} else if (place1 % 8 == 5) {
			if (Math.abs (place2 - place1) == 9 && place1 > place2) {
				return true;
			} else if (Math.abs (place2 - place1) == 7 && place1 < place2) {
				return true;
			}
		} else if (place1 % 8 == 1) {
			if (Math.abs (place2 - place1) == 7 && place1 < place2) {
				return true;
			} else if (Math.abs (place2 - place1) == 9 && place2 < place1) {
				return true;
			}
		}
		return false;
	}
	
	public boolean moveWraps (int place1, int place2) {
		if (place1 % 8 == 4 || place1 % 8 == 5) {
			if (Math.abs(place2 - place1) == 5 || Math.abs(place2 - place1) == 3) {
				return true;
			}
		} else {
			return false;
		}
		
		return false;
	}
	
	public boolean validMove (int place1, int place2, int playerNum) {
		Piece p = pieceAt (place1);
		
		if (p != null) {
		
			if (p.ownedBy (playerNum)) {
				Piece destination = pieceAt (place2);
				if (destination != null) {
					return false;
				}
			
				Piece jumped = jumpedPiece (place1, place2);
				if (jumped != null) {
					return true;
				}
				
				if (p.isKing()) {
					if (place1 % 8 > 4 || place1 % 8 == 0) {
						if (place1 - place2 == 4) {
							return true;
						} else if (place1 - place2 == 5) {
							return true;
						} else if (place1 - place2 == -3) {
							return true;
						} else if (place1 - place2 == -4) {
							return true;
						} else {
							return false;
						}
					} else {
						if (place1 - place2 == 4) {
							return true;
						} else if (place1 - place2 == 3) {
							return true;
						} else if (place1 - place2 == -5) {
							return true;
						} else if (place1 - place2 == -4) {
							return true;
						} else {
							return false;
						}
					}
				} else {
					if (playerNum == 0) {
						if (place2 < place1) {
							return false;
						}
					} else {
						if (place1 < place2) {
							return false;
						}
					}
					
					if (place1 < place2) {
						if (place1 % 8 > 4 || place1 % 8 == 0) {
							if (Math.abs(place1 - place2) == 4) {
								return true;
							}
							if (Math.abs(place1 - place2) == 3) {
								return true;
							}
						} else {
							if (Math.abs(place1 - place2) == 5) {
								return true;
							}
							if (Math.abs(place1 - place2) == 4) {
								return true;
							}
						}
					} else {
						if (place1 % 8 > 4 || place1 % 8 == 0) {
							if (Math.abs(place1 - place2) == 4) {
								return true;
							}
							if (Math.abs(place1 - place2) == 5) {
								return true;
							}
						} else {
							if (Math.abs(place1 - place2) == 3) {
								return true;
							}
							if (Math.abs(place1 - place2) == 4) {
								return true;
							}
						}
					}							
					
				}
			} else {
				return false;
			}
		} else {
			return false;
		}
		
		return false;
	}
	
	public Piece jumpedPiece (int place1, int place2) {
		Piece myPiece = (Piece) pieces.get (new Integer (place1));
		Piece p = (Piece) pieces.get (new Integer (place2));
		if (p != null) {
			return null;
		} else {
			if (Math.abs(place1 - place2) == 9) {
				if (place1 < place2) {
					if ((place1 % 8 > 4) || place1 % 8 == 0) {
						p = (Piece) pieces.get (new Integer(place1+4));
						if (p != null && !p.ownedBy (myPiece.getOwner())) {
							return p;
						} else {
							return null;
						}
					} else {
						p = (Piece) pieces.get (new Integer(place1+5));
						if (p != null && !p.ownedBy (myPiece.getOwner())) {
							return p;
						} else {
							return null;
						}
					}
				} else {
					if ((place1 % 8 > 4) || place1 % 8 == 0) {
						p = (Piece) pieces.get (new Integer(place1-5));
						if (p != null && !p.ownedBy (myPiece.getOwner())) {
							return p;
						} else {
							return null;
						}
					} else {
						p = (Piece) pieces.get (new Integer(place1-4));
						if (p != null && !p.ownedBy (myPiece.getOwner())) {
							return p;
						} else {
							return null;
						}
					}
				}
			} else if (Math.abs(place1 - place2) == 7) {
				if (place1 < place2) {
					if ((place1 % 8 > 4) || place1 % 8 == 0) {
						p = (Piece) pieces.get (new Integer(place1+3));
						if (p != null && !p.ownedBy (myPiece.getOwner())) {
							return p;
						} else {
							return null;
						}
					} else {
						p = (Piece) pieces.get (new Integer(place1+4));
						if (p != null && !p.ownedBy (myPiece.getOwner())) {
							return p;
						} else {
							return null;
						}
					}
				} else {
					if ((place1 % 8 > 4) || place1 % 8 == 0) {
						p = (Piece) pieces.get (new Integer(place1-4));
						if (p != null && !p.ownedBy (myPiece.getOwner())) {
							return p;
						} else {
							return null;
						}
					} else {
						p = (Piece) pieces.get (new Integer(place1-3));
						if (p != null && !p.ownedBy (myPiece.getOwner())) {
							return p;
						} else {
							return null;
						}
					}
				}
			}
		}
		return null;
	}
	
	public void addPiece (Piece p) {
		pieces.put (new Integer(p.getLocation()), p);
	}
	
	public void removePiece (Piece p) {
		pieces.remove (new Integer (p.getLocation()));
	}
	
	public Piece pieceAt (int i) {
		return (Piece) pieces.get (new Integer (i));
	}
	
	public void movePiece (Piece p, String move) {
		String[] ms = move.split("-");
		
		int start = new Integer (ms[0]).intValue();
		int end = new Integer (ms[1]).intValue();
		
		removePiece (p);
		p.movePiece (end);
		addPiece (p);
	}
	
	public void output () {

		System.out.println();
		System.out.println();
		

		String line1 = "";
		String line2 = "";
		String line3 = "";
		String line4 = "";

		boolean flip = false;
		
		for (int i=1; i<33; i++) {
			if (!flip) {
				line1 += "XXXX";
				line2 += "XXXX";
				line3 += "XXXX";
				line4 += "XXXX";
			}
			Piece p = (Piece) pieces.get(new Integer(i));
			
			String iVal = String.valueOf (i);
			if (iVal.length() < 2) {
				iVal = " " + iVal;
			}
			
			if (p != null) {
				if (p.ownedBy(0)) {
					line1 += "    ";
					line2 += " " + iVal + " ";
					if (p.isKing()) {
						line3 += " WK ";
					} else {
						line3 += " W  ";					
					}
					line4 += "    ";
				} else {
					line1 += "    ";
					line2 += " " + iVal + " ";
					if (p.isKing()) {
						line3 += " RK ";
					} else {
						line3 += " R  ";					
					}
					line4 += "    ";
				}
			} else {
				line1 += "    ";
				line2 += " " + iVal + " ";
				line3 += "    ";
				line4 += "    ";
			}
			
			if (flip) {
				line1 += "XXXX";
				line2 += "XXXX";
				line3 += "XXXX";
				line4 += "XXXX";
			}
			
			if (i % 4 == 0) {
				System.out.println (line1);
				System.out.println (line2);
				System.out.println (line3);
				System.out.println (line4);
				line1 = "";
				line2 = "";
				line3 = "";
				line4 = "";
				flip = !flip;
			}
		}
	}
}
